/*
**  TTL is a complete, if somewhat limited, terminal emulation
**  program  designed to demonstrate the use of the LiteComm
**  ToolBox.  The executable version is included so that you can
**  try it out while viewing the code.  To successfully create a
**  new version of TTL, you must have the XMODEM and WXMODEM engines
**  which are provided as part of your registration package.
**  While non-registered users cannot create a new version of TTL,
**  you may still compile the TTL program.  All required header files
**  are included in the shareware archive
**
**  Information Technology, Ltd.
*/

#include "litecomm.h"
#include "litexm.h"

#ifdef DLC
#include <dos.h>
#include <disp.h>
#include <stdio.h>

FILE _iob[_NFILE] =
{
	{0,0,0,_IOTRAN | _IOREAD,0,1},		/* stdin	*/
	{0,0,0,_IOTRAN | _IOWRT | _IONBF ,1,1},	/* stdout	*/
	{0,0,0,_IOTRAN | _IOWRT | _IONBF ,2,1},	/* stderr	*/
	{0,0,0,_IOTRAN | _IORW  ,3,1},		/* stdaux	*/
	{0,0,0,_IOTRAN | _IOWRT ,4,1},		/* stdprn	*/

	/* the rest have all 0 entries	*/
};
#endif

#ifdef __TURBOC__
#include <stdlib.h>
#include <fcntl.h>
#include <\tc\sys\stat.h>
#include <io.h>
#include <conio.h>
#include <mem.h>
#endif

#ifdef M_I86
#include "signal.h"
#include "conio.h"
#include "ctype.h"
#include "bios.h"
#include "fcntl.h"
#include "stdio.h"
#include <sys\types.h>
#include <sys\stat.h>
#endif

#define CTRLX 0x18

int nocbrk();
void xupload(void);
void xdnload(void);
void wupload(void);
void wdnload(void);

#ifdef DLC
#define putch(c) bdos(6,c)
#define cprintf printf
#define SEEK_SET 0

void cgets(char *);
void cputs(char *);
#endif

#ifndef M_I86
#define _KEYBRD_READ        0
#define _KEYBRD_READY       1
#define _bios_keybrd(x) bioskey(x)
#endif

/*
**  Comm parameters - human-readable form
*/
int baud = 2400;
char parity = 'N';
int data = 8;
int stop = 1;

/*
**  Comm parameters - Litecomm form
*/
unsigned pbaud = B2400;
unsigned pparity = NPARITY;
unsigned pbits = BIT8;
unsigned pstop = STOP1;

unsigned port = 3;

int xmode = 0;                          /* xmodem type */
int halfd = 0;                          /* half-duplex */
int hostm = 0;                          /* host mode */
int ctlc_hit = FALSE;


main()
{
	char    opt;

#ifdef M_I86
	signal(SIGINT, nocbrk);               /* set Ctrl-Break handler */
#endif

#ifdef __TURBOC__
    ctrlbrk(nocbrk);
#endif

#ifdef DLC
	int_intercept(0x23,nocbrk,100);
#endif


	if (comm_opn(port,B2400,NPARITY,BIT8,STOP1,1024,1024) == -1)
	{
		cputs("Error opening port\r\n");
		abort();
	}
	lc_setmdm(port, (DTR | RTS));

	while (1)
	{
		clrscr();
		cputs("-- MAIN MENU --\r\n");
		cputs("T - enter Terminal mode\r\n");
		cputs("    Ctrl-X leaves terminal mode\r\n");
		cprintf("H - toggle Host mode (now %s)\r\n",hostm ? "ON":"OFF");
		cprintf("G - toGgle half-duplex mode (now %s)\r\n",halfd ? "ON":"OFF");
		cprintf("C - change Comm parameters (now %d,%c,%d,%d)\r\n",
				baud,parity,data,stop);
		cprintf("X - change Xmodem mode (now %s)\r\n",xmode ? "WIN":"NOR");
		cputs("S - Send a file\r\n");
		cputs("R - Receive a file\r\n");
		cputs("Q - Quit\r\n\r\n");

		if ((opt = getch()) == 0)
			opt = getch();
		opt = toupper(opt);

		switch (opt)
		{
			case 'T':   terminal();
						break;
			case 'H':   if (hostm)
							hostm = 0;
						else
						{
							hostm = 1;
							halfd = 0;
						}
						break;
			case 'G':   if (halfd)
							halfd = 0;
						else
						{
							halfd = 1;
							hostm = 0;
						}
						break;
			case 'X':   if (xmode)
							xmode = 0;
						else
							xmode = 1;
						break;
			case 'S':   send();
						break;
			case 'R':   recv();
						break;
			case 'C':   chgcomm();
						break;
		}
		if (opt == 'Q')
			break;                      /* shut down time */
	}                                   /* while (1) */

	comm_close(port);
	exit(0);
}                                       /* main */

int nocbrk()
{
#ifdef M_I86
	signal(SIGINT, SIG_IGN);             /* set Ctrl-Break handler */
#endif
    ctlc_hit = TRUE;
    _abort_flag = TRUE;
#ifdef M_I86
	signal(SIGINT, nocbrk);             /* set Ctrl-Break handler */
#endif
    return(TRUE);
}

clrscr()
{
	int i;

	for (i=0; i<25; i++)
		cputs("\r\n");
}

terminal()
{
	unsigned ch;

	clrscr();

	while (1)
	{
		if ((ch = lc_get(port)) != -1)
		{
			putch(ch);
			if (hostm)
			{
				lc_put(port,ch);            /* echo back */
				if (ch == '\r')             /* was it return */
				{
					lc_put(port,'\n');      /* echo lf as well */
					putch('\n');
				}
			}
		}
		if (_bios_keybrd(_KEYBRD_READY) & 0xff)  /* anything typed */
		{
			ch = _bios_keybrd(_KEYBRD_READ) & 0xff; /* get input */
			if (ch == CTRLX)
				return(0);
			lc_put(port,ch);                /* xmit the char */
			if (hostm || halfd)             /* local echo needed ? */
			{
				putch(ch);
				if (ch == '\r')             /* was it CR */
				{
					putch('\n');            /* add LF */
					if (hostm)
						lc_put(port, '\n'); /* and send LF in host mode */
				}
			}
		}
	}                                       /* while */
}

chgcomm()
{
	char opt;
	unsigned sbaud;
	unsigned sparity;
	unsigned sbits;
	unsigned sstop;
	int hbaud;
	char hparity;
	int hdata;
	int hstop;

/*
** get current default settings
*/
	sbaud = pbaud;
	sparity = pparity;
	sbits = pbits;
	sstop = pstop;
	hbaud = baud;
	hparity = parity;
	hdata = data;
	hstop = stop;

	clrscr();

	while (1)
	{
		cputs("-- COMM PARAMETERS --\r\n");
		cprintf("  (Presently %d,%c,%d,%d)\r\n",
				hbaud,hparity,hdata,hstop);
		cputs("B - change Baud Rate\r\n");
		cputs("P - change Parity\r\n");
		cputs("D - change Data bits\r\n");
		cputs("S - change Stop bits\r\n\r\n");
		cputs("A - Abandon Changes\r\n");
		cputs("Q - Quit to Main Menu\r\n\r\n");

		if ((opt = getch()) == 0)
			opt = getch();
		opt = toupper(opt);

		switch (opt)
		{
			case 'A':
				return;
			case 'Q':
				pbaud = sbaud;          /* reset the globals */
				pparity = sparity;
				pbits = sbits;
				pstop = sstop;
				baud = hbaud;
				parity = hparity;
				data = hdata;
				stop = hstop;
				comm_setup(port,pbaud,pparity,pbits,pstop);
				return;
			case 'B':
				cputs("1 - 110, 2 - 300, 3 - 600, 4 - 1200, 5 - 2400\r\n");
				cputs("6 - 4800, 7 - 9600, 8 - 19200\r\n");
				if ((opt = getch()) == 0)
					opt = getch();
				switch (opt)
				{
					case '1': hbaud = 110;
							  sbaud = B110;
							  break;
					case '2': hbaud = 300;
							  sbaud = B300;
							  break;
					case '3': hbaud = 600;
							  sbaud = B600;
							  break;
					case '4': hbaud = 1200;
							  sbaud = B1200;
							  break;
					case '5': hbaud = 2400;
							  sbaud = B2400;
							  break;
					case '6': hbaud = 4800;
							  sbaud = B4800;
							  break;
					case '7': hbaud = 9600;
							  sbaud = B9600;
							  break;
					case '8': hbaud = 19200;
							  sbaud = B19200;
							  break;
				}
				break;
			case 'P':
				cputs("1 - NONE, 2 - EVEN, 3 - ODD, 4 - MARK, 5 - SPACE\r\n");
				if ((opt = getch()) == 0)
					opt = getch();
				switch (opt)
				{
					case '1': hparity = 'N';
							  sparity = NPARITY;
							  break;
					case '2': hparity = 'E';
							  sparity = EPARITY;
							  break;
					case '3': hparity = 'O';
							  sparity = OPARITY;
							  break;
					case '4': hparity = 'M';
							  sparity = MPARITY;
							  break;
					case '5': hparity = 'S';
							  sparity = SPARITY;
							  break;
				}
				break;
			case 'D':
				cputs("Number of Data Bits (5, 6, 7, 8)\r\n");
				if ((opt = getch()) == 0)
					opt = getch();
				switch (opt)
				{
					case '5': hdata = 5;
							  sbits = BIT5;
							  break;
					case '6': hdata = 6;
							  sbits = BIT6;
							  break;
					case '7': hdata = 7;
							  sbits = BIT7;
							  break;
					case '8': hdata = 8;
							  sbits = BIT8;
							  break;
				}
				break;
			case 'S':
				cputs("Number of Stop Bits (1, 2)\r\n");
				if ((opt = getch()) == 0)
					opt = getch();
				switch (opt)
				{
					case '1': hstop = 1;
							  sstop = STOP1;
							  break;
					case '2': hstop = 2;
							  sstop = STOP2;
							  break;
				}
				break;
		}
	}
}


send()
{
	if (xmode)
		wupload();
	else
		xupload();
}

recv()
{
	if (xmode)
		wdnload();
	else
		xdnload();
}

void xupload(void)
{
    char fname[14];
    int  fd;
    unsigned char buf[128];
    unsigned char hdshk;
    int hmode;
    int result;

    ctlc_hit = FALSE;
    cputs("File Name to Send: ");
    fname[0] = (char) 11;
    cgets(fname);
    cputs("\n");
    if (ctlc_hit == TRUE)
    {
        ctlc_hit = FALSE;
        return;
    }
    if ((fname[1] == 0) || (fname[2] == 0x00))
        return;

#ifndef DLC
    if ((fd = open(&fname[2],(O_RDONLY|O_BINARY))) == -1)
#else
    if ((fd = open(&fname[2],O_RDONLY)) == -1)
#endif
    {
        cputs("Unable to open the file\n");
        return;
    }
/*
** the file has been opened successfully...now switch the
** comport mode to 8 bits for xmodem protocol
*/
    if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
    {
        cputs("Unable to switch line mode\n");
        return;
    }
/*
** begin file transmission to receiver -
** in receiver's specified mode
*/
    while (TRUE)
    {
		memset(buf, 0x00, 128);     /* clear buff, short rec */
        if (read(fd, buf, sizeof(buf)) < 1)  /* EOF or Error */
        {
            lcxteot(3);            /* send end of file */
            cputs("\r\nEnd of Transmission\r\n");
            break;
        }
        result = lcxtrec(3, buf);
        switch(result)           /* what action to take ? */
        {
            case SUCCESS:
                printf("Sent record: %d\r",(rec-1));
                break;
            case CAN:
                cputs("\r\nCancel Received\r\n");
                break;
            case RETRIES:
                printf("\nToo Many tries...Rec %d\n",(rec-1));
                break;
            default:
                cputs("\r\nFatal transmission error\r\n");
                break;
        }
        if (result != SUCCESS)
            break;
    }                                          /* while TRUE */
    comm_setup(port,pbaud,pparity,pbits,pstop);
    close(fd);                                 /* close down the file */
}

void xdnload(void)
{
    char fname[14];
    int  fd;
    unsigned char buf[128];
    unsigned char hdshk;
    unsigned int bsize;
    int hmode;
    int result;
    char wstr[6];

    ctlc_hit = FALSE;
    cputs("File Name to Receive: ");
    fname[0] = (char) 11;
    cgets(fname);
    cputs("\n");
    if (ctlc_hit == TRUE)
    {
        ctlc_hit = FALSE;
        return;
    }
    if ((fname[1] == 0) || (fname[2] == 0x00))
        return;

#ifndef DLC
    if ((fd = open(&fname[2],
         (O_WRONLY|O_CREAT|O_TRUNC|O_BINARY),(S_IREAD|S_IWRITE))) == -1)
#else
	if ((fd = creat(&fname[2], 0)) == -1)
#endif
    {
        cputs("Unable to open the file\n");
        return;
    }
/*
** the file has been opened successfully...now switch the
** comport mode to 8 bits for xmodem protocol
*/
    if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
    {
        cputs("Unable to switch line mode\n");
        return;
    }
/*
** setup conditions for XMODEM receive
*/
    hdshk = CRC;                 /* use crc method */
    hmode = RELAXED;             /* normal time delay used */
    while (TRUE)
    {
        while ((result = lcxrrec(3, buf, hmode, &hdshk)) == SUCCESS)
        {
            write(fd, buf, sizeof(buf));  /* dump the block */
			cputs("Received record: ");
			itoa((rec-1),wstr,10);
			cputs(wstr);
            cputs("\r");
        }
        switch(result)           /* unusual conditions */
        {
            case DUPSEQ:
                cputs("\r\nDuplicate record seq\r\n");
                break;
            case CAN:
                cputs("\r\nCancelled by host\r\n");
                break;
            case EOT:
                cputs("\r\nNormal Termination\r\n");
                break;
            case TOUT:
                cputs("SOH timeout\r\n");
                break;
            case RETRIES:
                printf("\nToo Many tries...Rec %d\n",(rec-1));
                break;
            default:
                cputs("\r\nFatal transmission error\r\n");
                break;
        }
        if ((result != SUCCESS) && (result != DUPSEQ))
            break;
    }                                          /* while TRUE */

    comm_setup(port,pbaud,pparity,pbits,pstop);
    close(fd);                                 /* close down the file */
}

void wupload(void)
{
    char fname[14];
    int  fd;
    unsigned char buf[128];
    unsigned char hdshk;
    int hmode;
    int result;
    int nrec;                           /* returned record */

    ctlc_hit = FALSE;
    cputs("File Name to Send: ");
    fname[0] = (char) 11;
    cgets(fname);
    cputs("\n");
    if (ctlc_hit == TRUE)
    {
        ctlc_hit = FALSE;
        return;
    }
    if ((fname[1] == 0) || (fname[2] == 0x00))
        return;

#ifndef DLC
    if ((fd = open(&fname[2],(O_RDONLY|O_BINARY))) == -1)
#else
    if ((fd = open(&fname[2],O_RDONLY)) == -1)
#endif
    {
        cputs("Unable to open the file\n");
        return;
    }
/*
** the file has been opened successfully...now switch the
** comport mode to 8 bits for xmodem protocol
*/
    if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
    {
        cputs("Unable to switch line mode\n");
        return;
    }
    lc_xoff(3,TRUE);                    /* turn on auto xon-xoff */

/*
** begin file transmission to receiver -
** in receiver's specified mode
*/
    while (TRUE)
    {
        memset(buf, 0x00, 128);     /* clear buff, short rec */
        if (read(fd, buf, sizeof(buf)) < 1)  /* EOF or Error */
            nrec = -1;              /* defines End of File */
        else
            nrec = 0;

        result = lwxtrec(3, buf, &nrec);

        switch(result)           /* what action to take ? */
        {
            case SUCCESS:
                if (nrec == -1)  /* EOF confirmed */
                    cputs("\r\nEnd of Transmission\r\n");
                else
                    printf("Sent record: %d\r",rec);
                break;
            case CAN:
                cputs("\r\nCancel Received\r\n");
                break;
            case RETRIES:
                printf("\nToo Many tries...Rec %d\n",rec);
                break;
            case RESEND:                /* must reset file position */
                lseek(fd, (long)(nrec * sizeof(buf)), SEEK_SET);
                break;
            default:
                cputs("\r\nFatal transmission error\r\n");
                break;
        }
        if (result == RESEND)
            continue;
        if (result == SUCCESS)
            if (nrec == -1)             /* was EOF signalled */
                break;
            else
                continue;
        break;                          /* some other error */
    }                                          /* while TRUE */
    lc_xoff(3, FALSE);                         /* turn off auto xon/xoff */
    comm_setup(port,pbaud,pparity,pbits,pstop);
    close(fd);                                 /* close down the file */
}

void wdnload(void)
{
    char fname[14];
    int  fd;
    unsigned char buf[128];
    unsigned char hdshk;
    int hmode;
    int result;
    char wstr[6];

    ctlc_hit = FALSE;
    cputs("File Name to Receive: ");
    fname[0] = (char) 11;
    cgets(fname);
    cputs("\n");
    if (ctlc_hit == TRUE)
    {
        ctlc_hit = FALSE;
        return;
    }
    if ((fname[1] == 0) || (fname[2] == 0x00))
        return;

#ifndef DLC
    if ((fd = open(&fname[2],
         (O_WRONLY|O_CREAT|O_TRUNC|O_BINARY),(S_IREAD|S_IWRITE))) == -1)
#else
	if ((fd = creat(&fname[2], 0)) == -1)
#endif
    {
        cputs("Unable to open the file\n");
        return;
    }
/*
** the file has been opened successfully...now switch the
** comport mode to 8 bits for xmodem protocol
*/
    if (comm_setup(port,pbaud,NPARITY,BIT8,pstop) == ERR)
    {
        cputs("Unable to switch line mode\n");
        return;
    }
/*
** setup conditions for XMODEM receive
*/
    while (TRUE)
    {
        while ((result = lwxrrec(3, buf)) == SUCCESS)
        {
            write(fd, buf, sizeof(buf));  /* dump the block */
            cputs("Received record: ");
            itoa((rec-1),wstr,10);
            cputs(wstr);
            cputs("\r");
        }
        switch(result)           /* unusual conditions */
        {
            case DUPSEQ:
                cputs("\r\nDuplicate record seq\r\n");
                break;
            case CAN:
                cputs("\r\nCancelled by host\r\n");
                break;
            case EOT:
                cputs("\r\nNormal Termination\r\n");
                break;
            case TOUT:
                cputs("SOH timeout\r\n");
                break;
            case RETRIES:
                printf("\nToo Many tries...Rec %d\n",(rec-1));
                break;
            default:
                cputs("\r\nFatal transmission error\r\n");
                break;
        }
        if ((result != SUCCESS) && (result != DUPSEQ))
            break;
    }                                          /* while TRUE */

    comm_setup(port,pbaud,pparity,pbits,pstop);
    close(fd);                                 /* close down the file */
}

#ifdef DLC

int bioskey(func)
int func;
{
	REGS	iregs;
	REGS	oregs;
	unsigned rflag;

	iregs.ax = func << 8;			/* status check */
	rflag = int86(0x16, &iregs, &oregs);	/* call keyboard */
	switch (func)
	{
		case 0: return(oregs.ax);
				break;
		case 1: if (rflag & 0x40)			/* test zero flag */
					return(0);
				return(1);					/* says character waits */
				break;
	}
	return(0);
}										/* bioskey */

void cgets(s)
char *s;
{
	bdosx(10,s);				/* call dos function */
	s[2 + (int) s[1]] = 0x00;	/* kill imbedded <CR> */
}

void cputs(s)
char *s;
{
	char x[81];

	strcpy(x,s);
	strcat(x,"$");
	bdosx(9,x);
}

#endif
